<?php
 /**
 * NOTICE OF LICENSE 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author    MigrationPro
 * @copyright Copyright (c) 2012-2023 MigrationPro
 * @license   http://www.apache.org/licenses/LICENSE-2.0
 * @package   MigrationPro: OpenCart to PrestaShop Migrate tool
 */

 
if (!defined('_PS_VERSION_')) {
    exit;
}
class WooLinkRewrite extends ObjectModel
{
    const TYPE_CATEGORY = 'c';
    const TYPE_PRODUCT = 'p';

    public $id_data;
    public $entity_type;
    public $source_id;
    public $local_id;

    public static $definition = array(
        'table' => 'woomigrationpro_link_rewrite',
        'primary' => 'id_data',
        'fields' => array(
            'entity_type' => array('type' => self::TYPE_STRING, 'validate' => 'isGenericName', 'required' => true),
            'rewrite_url' => array('type' => self::TYPE_STRING),
            'source_id' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt', 'required' => true),
            'local_id' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt', 'required' => true)
        ),
    );

    /**
     * @param $entity_type
     * @param $rewrite_url
     * @param $source_id
     * @param $local_id
     * @return bool
     */
    public static function import($entity_type, $rewrite_url, $source_id, $local_id)
    {
        $entity_type = constant("self::TYPE_" . Tools::strtoupper($entity_type));
        $sql = 'INSERT INTO ' . _DB_PREFIX_ . 'woomigrationpro_link_rewrite SET entity_type=\'' . pSQL($entity_type) . '\', rewrite_url=\'' . pSQL($rewrite_url) . '\', source_id=' . (int)$source_id . ', local_id=' . (int)$local_id . ' ON DUPLICATE KEY UPDATE local_id=' . (int)$local_id;

        return Db::getInstance()->execute($sql);
    }

    /**
     * @param $entity_type
     * @param $sourceID
     * @return false|string|null
     */
    public static function getLocalID($entity_type, $sourceID)
    {
        $entity_type = constant("self::TYPE_" . Tools::strtoupper(pSQL($entity_type)));

        $sql = 'SELECT keyword FROM ' . _DB_PREFIX_ . 'woomigrationpro_link_rewrite WHERE entity_type=\'' . pSQL($entity_type) . '\' AND source_id=' . (int)$sourceID . ';';

        return Db::getInstance()->getValue($sql);
    }
}
